with Ada.Characters.Latin_1;
with Ada.Command_Line;
with Ada.Text_IO;
with POSIX.Process_Primitives;
with UStrings;

-- ANSI terminal control: http://www.termsys.demon.co.uk/vtansi.htm
-- UStrings:              From the Debian/Ubuntu package "adacgi"

procedure More is
   Page_Length : constant Positive := 23;

   Terminal : Ada.Text_IO.File_Type;

   type Jumps is (Quit, Next_File, More_Lines);

   type Jump (Mode : Jumps := More_lines) is
      record
         case Mode is
            when Quit | Next_File => null;
            when More_Lines       => Count : Positive;
         end case;
      end record;

   function See_More_Lines return Jump is
      use Ada.Text_IO;
      package Latin_1 renames Ada.Characters.Latin_1;
      Key : Character;
   begin
      Put (Latin_1.ESC & "[7m more? " & Latin_1.ESC & "[m");
      loop
         Get_Immediate (File => Terminal,
                        Item => Key);
         Put (Latin_1.CR & Latin_1.ESC & "[K");
         case Key is
            when 'q' | 'Q' =>
               return (Mode => Quit);
            when 'n' | 'N' =>
               return (Mode => Next_File);
            when ' ' =>
               return (Mode => More_Lines, Count => Page_Length);
            when Latin_1.LF =>
               return (Mode => More_Lines, Count => 1);
            when others =>
               null;
         end case;
      end loop;
   end See_More_Lines;

   Lines_To_Copy : Natural := Page_Length;

   procedure Copy_Line (Text : in     String;
                        Done : in out Boolean) is
      More : Jump;
   begin
      if not Done then
         if Lines_To_Copy = 0 then
            More := See_More_Lines;

            case More.Mode is
               when Quit =>
                  POSIX.Process_Primitives.Exit_Process (Status => 0);
               when Next_File =>
                  Done := True;
                  return;
               when More_Lines =>
                  Lines_To_Copy := More.Count;
            end case;
         end if;

         Ada.Text_IO.Put_Line (Text);
         Lines_To_Copy := Lines_To_Copy - 1;
      end if;
   end Copy_Line;

   procedure Copy_Line (Input : in     Ada.Text_IO.File_Type;
                        Done  : in out Boolean) is
      use UStrings;
      Line : UString;
   begin
      Get_Line (File => Input,
                Item => Line);
      Copy_Line (Text => S (Line),
                 Done => Done);
   end Copy_Line;

   procedure Do_More (File : in     Ada.Text_IO.File_Type) is
      use Ada.Text_IO;
      Done : Boolean := False;
   begin
      while not Done and not End_Of_File (File) loop
         Copy_Line (Input => File,
                    Done  => Done);
      end loop;
   end Do_More;
begin
   Ada.Text_IO.Open (File => Terminal,
                     Name => "/dev/tty",
                     Mode => Ada.Text_IO.In_File);

   if Ada.Command_Line.Argument_Count = 0 then
      Do_More (File => Ada.Text_IO.Standard_Input);
   else
      for Index in 1 .. Ada.Command_Line.Argument_Count loop
         declare
            use Ada.Text_IO;
            Input : File_Type;
            Done  : Boolean := False;
         begin
            Copy_Line (Text => "::::::::::::::",
                       Done => Done);
            Copy_Line (Text => Ada.Command_Line.Argument (Index),
                       Done => Done);
            Copy_Line (Text => "::::::::::::::",
                       Done => Done);
            if not Done then
               Open (File => Input,
                     Name => Ada.Command_Line.Argument (Index),
                     Mode => In_File);
               Do_More (File => Input);
               Close (File => Input);
            end if;
         end;
      end loop;
   end if;

   Ada.Text_IO.Close (File => Terminal);
end More;
