--  fileinfo.adb - use Get_File_Status to obtain and print file properties
--               - some members are just numbers

with Ada.Command_Line;
with Ada.Text_IO;
with POSIX.Calendar;
with POSIX.File_Status;
with POSIX.Permissions.Text_IO;
with POSIX.Process_Identification;

procedure File_Information is
   use POSIX.File_Status;
   procedure Show (Name : in     String;
                   Info : in     Status) is
      use Ada.Text_IO;
      use POSIX.Calendar;
      use POSIX.Permissions.Text_IO;
      use POSIX.Process_Identification;
   begin
      Put_Line ("   mode: " & Image (Permission_Set_Of (Info)));
      Put_Line ("  links: " & Link_Count_Of (Info)'Img);
      Put_Line ("   user: " & Image (Owner_Of (Info)));
      Put_Line ("  group: " & Image (Group_Of (Info)));
      Put_Line ("   size: " & Size_Of (Info)'Img);
      Put_Line ("modtime: " & POSIX.Get_Seconds
                  (To_Timespec (Last_Modification_Time_Of (Info)))'Img);
      Put_Line ("   name: " & Name);
   end Show;
begin
   if Ada.Command_Line.Argument_Count >= 1 then
      Show (Name => Ada.Command_Line.Argument (1),
            Info => Get_File_Status
              (POSIX.To_POSIX_String (Ada.Command_Line.Argument (1))));
   end if;
end File_Information;
