--  showtty.adb
--     displays some current tty settings

with Ada.Characters.Handling,
     Ada.Integer_Text_IO,
     Ada.Strings.Fixed,
     Ada.Strings.Maps,
     Ada.Text_IO,
     POSIX.IO,
     POSIX.Terminal_Functions;

use POSIX,
    POSIX.IO,
    POSIX.Terminal_Functions;

procedure Show_TTY is
   procedure Show_Baud_Rate (Speed : in     Baud_Rate) is
      use Ada.Text_IO;
   begin
      Put ("The baud rate is ");
      case Speed is
         when B300 =>
            Put_Line ("300");
         when B600 =>
            Put_Line ("600");
         when B1200 =>
            Put_Line ("1200");
         when B1800 =>
            Put_Line ("1800");
         when B2400 =>
            Put_Line ("2400");
         when B4800 =>
            Put_Line ("4800");
         when B9600 =>
            Put_Line ("9600");
         when others =>
            Put_Line ("high");
      end case;
   end Show_Baud_Rate;

   function Control (C : in     POSIX_Character) return Character is
   begin
      return Character'Val (Character'Pos ('A') - 1 + POSIX_Character'Pos (C));
   end Control;

   procedure Show_Character (Meaning : in     String;
                             Char    : in     POSIX_Character) is
      use Ada.Integer_Text_IO,
          Ada.Text_IO;
   begin
      Put (Item => "The " & Meaning & " is ASCII ");
      Put (Item => POSIX_Character'Pos (Char), Width => 0);
      Put (Item => ", Ctrl-" & Control (Char) & ".");
      New_Line;
   end Show_Character;

   procedure Show_Modes (Modes : in     Terminal_Modes_Set) is
      function Capitalise (Item : in String) return String is
         use Ada.Characters.Handling;
      begin
         return To_Upper (Item (Item'First)) &
                To_Lower (Item (Item'First + 1 .. Item'Last));
      end Capitalise;

      function Description (Mode : in     Terminal_Modes) return String is
         use Ada.Strings.Fixed,
             Ada.Strings.Maps;
      begin
         return Capitalise (Translate (Terminal_Modes'Image (Mode),
                                       To_Mapping ("_", " ")));
      end Description;
   begin
      for Mode in Modes'Range loop
         if Modes (Mode) then
            Ada.Text_IO.Put_Line ("  " & Description (Mode) & " is ON.");
         else
            Ada.Text_IO.Put_Line ("  " & Description (Mode) & " is OFF.");
         end if;
      end loop;
   end Show_Modes;

   Terminal : Terminal_Characteristics;
begin
   begin
      Terminal := Get_Terminal_Characteristics (File => Standard_Input);
   exception
      when others =>
         Ada.Text_IO.Put_Line
           (File => Ada.Text_IO.Standard_Error,
            Item => "Cannot get terminal characteristics for standard input.");
         return;
   end;

   Show_Baud_Rate (Speed => Output_Baud_Rate_Of (Terminal));
   Show_Character (Meaning => "erase character",
                   Char    => Special_Control_Character_Of (Terminal,
                                                            Erase_Char));
   Show_Character (Meaning => "line kill",
                   Char    => Special_Control_Character_Of (Terminal,
                                                            Kill_Char));
   Show_Modes (Modes => Terminal_Modes_Of (Terminal));
end Show_TTY;
