--  write1.adb
--
--  Purpose: Send messages to another terminal
--   Method: Open the other terminal for output, then copy from
--           Standard_Input to that terminal.
--    Shows: A terminal is just a file supporting regular I/O.
--    Usage: ./write0 ttyname

with Ada.Characters.Latin_1,
     Ada.Command_Line,
     Ada.Direct_IO,
     POSIX.Files,
     POSIX.IO,
     POSIX.Process_Primitives,
     POSIX.Strings,
     POSIX.UTMP;

use Ada.Command_Line,
    POSIX,
    POSIX.IO,
    POSIX.Process_Primitives;

procedure Write is
   function Get_TTY (User : in     String) return POSIX_String is
      package UTMP_IO is new Ada.Direct_IO (POSIX.UTMP.Instance);
      use POSIX.Files,
          POSIX.Strings,
          POSIX.UTMP,
          UTMP_IO;
      File : UTMP_IO.File_Type;
      Item : POSIX.UTMP.Instance;
   begin
      Open (File => File,
            Name => POSIX.UTMP.File_Name,
            Mode => In_File);

      while not End_Of_File (File) loop
         Read (File => File,
               Item => Item);

         if Item.Login_Type = User_Process then
            if User = Item.User then
               if Is_File_Present ("/dev/" & Item.Line) then
                  Close (File => File);
                  return "/dev/" & Item.Line;
               end if;
            end if;
         end if;
      end loop;

      Close (File => File);
      return "";
   end Get_TTY;

   Terminal      : File_Descriptor;
   Buffer        : IO_Buffer (1 .. 1024);
   Bytes_Read    : Natural;
   Bytes_Written : Natural;
begin
   if Argument_Count = 1 then
      declare
         TTY : POSIX_String := Get_TTY (User => Argument (1));
      begin
         Terminal := Open (Name => TTY,
                           Mode => Write_Only);
      exception
         when others =>
            NONSTANDARD_Write
              (File   => Standard_Error,
               Buffer => "Error opening file: '" & TTY & "'" & POSIX.LF,
               Last   => Bytes_Written);
            raise;
      end;
      loop
         NONSTANDARD_Read
           (File   => Standard_Input,
            Buffer => Buffer,
            Last   => Bytes_Read);
         NONSTANDARD_Write
           (File   => Terminal,
            Buffer => Buffer (Buffer'First .. Bytes_Read),
            Last   => Bytes_Written);
      end loop;
      Close (File => Terminal);
   else
      NONSTANDARD_Write
        (File   => Standard_Error,
         Buffer => "Usage: write1 logname" & POSIX.LF,
         Last   => Bytes_Written);
      Exit_Process (Status => 1);
   end if;
exception
   when others =>
      null;
end Write;
