-- play_again1.adb
--      purpose: ask if user wants another transaction
--       method: set tty into char-by-char mode, read char, return result
--      returns: yes => 0
--               no  => 1
--       better: do not echo inappropriate input

with
  Ada.Text_IO,
  POSIX.Direct_Character_IO,
  POSIX.IO,
  POSIX.Process_Primitives,
  POSIX.Terminal_Functions;

use
  POSIX.IO,
  POSIX.Process_Primitives,
  POSIX.Terminal_Functions;

procedure Play_Again is
   function Get_Response (Question : in     String) return Exit_Status is
      use
        Ada.Text_IO,
        POSIX.Direct_Character_IO;
      Input : Character;
   begin
      Put (Item => Question & " (y/n)? ");
      loop
         Input := To_Character (Get);
         case Input is
            when 'y' | 'Y' =>
               return 0;
            when 'n' | 'N' =>
               return 1;
            when others =>
               New_Line (File => Ada.Text_IO.Standard_Error);
               Put_Line (File => Ada.Text_IO.Standard_Error,
                         Item => "Cannot understand '" & Input &
                                 "'.  Please type 'y' or 'n'.");
         end case;
      end loop;
   end Get_Response;

   procedure Set_Character_By_Character_Mode is
      Info  : Terminal_Characteristics;
      Modes : Terminal_Modes_Set;
   begin
      Info := Get_Terminal_Characteristics (File => Standard_Input);
      Modes := Terminal_Modes_Of (Info);
      Modes (Canonical_Input) := False;
      Define_Terminal_Modes (Characteristics => Info,
                             Modes           => Modes);
      Define_Minimum_Input_Count (Characteristics     => Info,
                                  Minimum_Input_Count => 1);
      Set_Terminal_Characteristics (File            => POSIX.IO.Standard_Input,
                                    Characteristics => Info);
   end Set_Character_By_Character_Mode;

   TTY_Mode : Terminal_Characteristics;
   Question : constant String := "Do you want another transaction";
   Answer   : Exit_Status;
begin
   TTY_Mode := Get_Terminal_Characteristics (File => Standard_Input);
   Set_Character_By_Character_Mode;
   Answer := Get_Response (Question);
   Set_Terminal_Characteristics (File            => Standard_Input,
                                 Characteristics => TTY_Mode);
   Exit_Process (Status => Answer);
end Play_Again;
